/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

#define        DSET_AI          devAiK175Gpib
#define        DSET_BO          devBoK175Gpib
#define        DSET_MBBO        devMbboK175Gpib
#define        DSET_SI          devSiK175Gpib

#include        <devCommonGpib.h>
#include        <devGpib.h>

/*
 * Use the TIME_WINDOW defn to indicate how long commands should be ignored
 * for a given device after it times out.  The ignored commands will be
 * returned as errors to device support.
 *
 * Use the TIMEOUT to define how long you wish to wait for an I/O operation
 * to complete once started.
 */
#define TIME_WINDOW    3.0
#define TIMEOUT        1.5


/******************************************************************************
 *
 * String arrays for EFAST operations.  Note that the last entry must be 
 * NULL.
 *
 * On input operations, only as many bytes as are found in the string array
 * elements are compared.  If there are more bytes than that in the input
 * message, they are ignored.  The first matching string found (starting
 * from the 0'th element) will be used as a match.
 *
 * NOTE: For the input operations, the strings are compared literally!  This
 * can cause problems if the instrument is returning things like \r and \n
 * characters.  You must take care when defining input strings so you include
 * them as well.
 *
 ******************************************************************************/

static char     *(set175range[]) = {"R0X",  /* Auto Scale */
                                    "R1X",  /*  200 (mV) 200 (Ohm)  */
                                    "R2X",  /*    2 (V)    2 (kOhm) */
                                    "R3X",  /*   20 (V)   20 (kOhm) */
                                    "R4X",  /*  200 (V)  200 (kOhm) */
                                    "R5X",  /* 1000 (V)    2 (MOhm) */
                                    NULL};

/******************************************************************************
 *
 * Array of structures that define all GPIB messages
 * supported for this type of instrument.
 *
 ******************************************************************************/

/* forward declarations of some custom convert routines */

static struct gpibCmd gpibCmds[] = 
{
  /* Param 0, (model)   */
  FILL,

  /* Param 1 initialization string */
  FILL,

  /* Param 2 read value */
  /* ignore first 4 characters, convert floating point value */
  {&DSET_AI, GPIBREAD, IB_Q_HIGH, "", "%*4c%lf", 0, 32,
  NULL, 0, 0, NULL, NULL, NULL},

  /* Param 3 set function */
  FILL,

  /* Param 4 set range */
  {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
  NULL, 0, 0, set175range, NULL, NULL},
 
  /* Param 5 status word */
  {&DSET_SI, GPIBREAD, IB_Q_HIGH, "U0X", "%s", 0, 32,
  NULL, 0, 0, NULL, NULL, NULL},
};


/* The following is the number of elements in the command array above.  */
#define NUMPARAMS        sizeof(gpibCmds)/sizeof(struct gpibCmd)
/******************************************************************************
 *
 * Initialization for device support
 * This is called one time before any records are initialized with a parm
 * value of 0.  And then again AFTER all record-level init is complete
 * with a param value of 1.
 *
 ******************************************************************************/
static long
init_ai(int parm)
{
  if(parm==0)  {
    devSupParms.respond2Writes = -1;
    devSupParms.timeWindow = TIME_WINDOW;
    devSupParms.gpibCmds = gpibCmds;
    devSupParms.numparams = NUMPARAMS;
    devSupParms.name = "devXxK175Gpib";
    devSupParms.timeout = TIMEOUT;
  }
  return 0;
}
